/*       Copyrighted by Kenneth De Jong - 2005          */
/* Licensed under the Academic Free License version 2.1 */

/*
   Population: A simple EA population class
*/


import java.text.DecimalFormat;
import java.util.*;


class Population
  {
  int pop_size;

  Individual indiv[];

  Individual   local_max = new Individual();
  Individual   local_min = new Individual();

  float   local_ave = 0;

  EA ea;


/*
   Population constructor: creates randomly generated genomes
*/

Population(EA ea)
  {

  int  i;
  String size;

  this.ea = ea;
  this.pop_size = ea.pop_size;
  indiv = new Individual[pop_size];

  for (i=0; i<pop_size; i++)       /* create initial population */
    {
    indiv[i] = new Individual(++ea.birth_cnt, ea.rand);
    eval(indiv[i]);
    }
}



/*
  eval:
*/

void eval(Individual indiv)
  {

  indiv.eval();

  ea.global_ave = ea.global_ave + indiv.fitness;
  if (indiv.fitness > ea.global_max.fitness)  ea.global_max = indiv.copy();
  if (indiv.fitness < ea.global_min.fitness)  ea.global_min = indiv.copy();
  }


/*
   Select_uniform:  select an individual from population uniform randomly
*/

Individual  select_uniform()
  {
  int i;

  i = ea.rand.irand(pop_size);
  return (indiv[i]);
  }



/*
  Survival:  conditionally inserts child into the population
                 by replacing an existing member.
*/

void uniform_survival(Individual  child)
  {
  int i;

  i = ea.rand.irand(pop_size);
  if (child.fitness > indiv[i].fitness)  
    {
    indiv[i] = child;
    ea.death_cnt++;
    }
  }



DecimalFormat int4 = new DecimalFormat("###0");
DecimalFormat f12_5 = new DecimalFormat("#####0.00000");


void dump()
  {
  int i;

  local_max.fitness = Float.NEGATIVE_INFINITY;
  local_min.fitness = Float.POSITIVE_INFINITY;
  local_ave = 0;

  for (i=0; i<pop_size; i++)
     {
     local_ave = local_ave + indiv[i].fitness;

     if (indiv[i].fitness > local_max.fitness)  
         local_max = indiv[i].copy();

     if (indiv[i].fitness < local_min.fitness)  
         local_min = indiv[i].copy();
     }

  if (ea.report_pop_stats)
     System.out.println(
		Util.pad_left(int4.format(ea.birth_cnt),9) + 
		Util.pad_left(f12_5.format(local_max.fitness),15) +
		Util.pad_left(f12_5.format(local_ave/pop_size),15) + 
		Util.pad_left(f12_5.format(local_min.fitness),15) ) ;
  else
     System.out.println(
                " Local fitness:   max = " 
		+ Util.pad_left(f12_5.format(local_max.fitness),12) +
		", ave = " +
		Util.pad_left(f12_5.format(local_ave/pop_size),12) + 
                ", min = " +
		Util.pad_left(f12_5.format(local_min.fitness),12) );

  if (ea.report_pop_dump)
      {
      System.out.println(" Indiv  birthdate   fitness      gene values"); 

      for (i=0; i<pop_size; i++)
        {
        System.out.print(Util.pad_left(int4.format(i+1), 5) + 
	  Util.pad_left(int4.format(indiv[i].birthdate), 7) + 
	  Util.pad_left(f12_5.format(indiv[i].fitness), 18) );
          indiv[i].genome.print();

        System.out.println("");
        }
      }
  }

}











