/*       Copyrighted by Kenneth De Jong - 2005          */
/* Licensed under the Academic Free License version 2.1 */

/*
    RvpEA:  a class of simple evolutionary algorithms for Rvp Landscapes
*/

import java.util.*;
import java.text.DecimalFormat;


abstract class RvpEA   extends EA
    {
    static final int no_mutation = 0;
    static final int delta_mutation = 1;
    static final int gaussian_mutation = 2;

    int  mutation_type = delta_mutation;
    float mutation_step_size = 0.1F;


/*
    RvpEA constructor
*/

RvpEA(Properties props, RvpLscape rls, int rand_seed)
    {
    super(rand_seed);

    Genome.map_genome(rls);
    global_max.fitness = Float.NEGATIVE_INFINITY;
    global_min.fitness = Float.POSITIVE_INFINITY;

    String mutation_options;
    String mutation_type;
    StringTokenizer  st;

    mutation_options = props.getProperty("Mutation");
    if (mutation_options == null) {
	Util.ec_warn("mutation type unspecified; default used.");
	mutation_type = "delta";
        }
    else {
	st = new StringTokenizer(mutation_options," ");
	mutation_type = st.nextToken();
	if (mutation_type.equals("gaussian"))
	    this.mutation_type = gaussian_mutation;
	else if (mutation_type.equals("delta"))
	    this.mutation_type = delta_mutation;
	else if (mutation_type.equals("none"))
	    this.mutation_type = no_mutation;
	else
	    Util.ec_error("unknown mutation type: " + mutation_type);

	if (! mutation_type.equals("none")) {
	    if (st.hasMoreTokens()) 
		mutation_step_size = Float.parseFloat(st.nextToken());
	    else  Util.ec_warn("no mutation step size specified; default used.");
	    }
        }

    if (! mutation_type.equals("none"))
	System.out.println("  Using " + mutation_type + 
                       " mutation with step size " + mutation_step_size);

    EA_init(props);    // finish superclass initialization
    }



DecimalFormat int4 = new DecimalFormat("###0");
DecimalFormat f12_6 = new DecimalFormat("#####0.000000");

/*
   Print_best:
*/

void print_best()
  {
  System.out.println("\nGlobal best:  birthdate   fitness      gene values"); 

  System.out.print("           " + 
	  Util.pad_left(int4.format(global_max.birthdate), 7) + 
	  Util.pad_left(f12_6.format(global_max.fitness), 16) );
          global_max.genome.print();

  System.out.println("");
  }

}
