/*       Copyrighted by Kenneth De Jong - 2005          */
/* Licensed under the Academic Free License version 2.1 */

/*
   BinaryGenome:   a simple binary genome representation for RVP Landscapes
*/


import java.text.DecimalFormat;


abstract class BinaryGenome extends Genome
    {
    static double param_res[];
    static int    param_len[];

    int gene[];


/*
   Genome constructor: uses a uniform random number generator to assign
                initial binary values to each gene.
*/

BinaryGenome(EC_rand rand)
  {
  int  i;

  gene = new int[gene_cnt];

  for (i=0; i<gene_cnt; i++)   gene[i] = rand.irand(2);
  }



/*
   Genome Constructor: allocates uninitialized genes
*/

BinaryGenome()
  {

  gene = new int[gene_cnt];

  }




/*
   BinaryGenomeInit: initializes the genome/phenome mapping for entire class
*/

static void BinaryGenomeInit(RvpLscape lscape) {

  int i;

  rls = lscape;

  gene_cnt = 0;
  param_len = new int[rls.param_cnt];
  param_res = new double[rls.param_cnt];

  for (i=0; i<rls.param_cnt; i++) 
      {
      param_len[i] = 
	  (int) log2( (rls.init_param_ubd[i] - rls.init_param_lbd[i]) / 
		      rls.init_param_res[i]) + 1;
      gene_cnt += param_len[i];
      param_res[i] = (rls.init_param_ubd[i]-rls.init_param_lbd[i]) / 
	  Math.pow(2, param_len[i]);
      }

  System.out.println("  Using a genome with " + gene_cnt +
		     " binary genes.");
  }


/*
   eval: computes the fitness of a given genome
*/

float eval()
  {
  int i, j, k, int_val;
  float param_val[] = new float[rls.param_cnt];

  k=0;
  for (i=0; i< rls.param_cnt; i++)
    {
    int_val = 0;
    for (j=0; j<param_len[i]; j++)
	  int_val = (2*int_val) + gene[k++];

    param_val[i] = (float) (rls.init_param_lbd[i] + int_val*param_res[i]);
    }

  return (rls.eval(param_val));
  }




DecimalFormat f10_5 = new DecimalFormat("###0.00000");


/*
   print_genome:  prints the binary gene values
*/

void print()
  {
  int i, j, k, int_val;

              // print binary representation

  for (i=0; i<gene_cnt; i++)        
        System.out.print(gene[i]);


              // print real-valued representation

  float param_val[] = new float[rls.param_cnt];

  k=0;
  for (i=0; i< rls.param_cnt; i++)
    {
    int_val = 0;
    for (j=0; j<param_len[i]; j++)
	  int_val = (2*int_val) + gene[k++];

    param_val[i] = (float) (rls.init_param_lbd[i] + int_val*param_res[i]);
    System.out.print("  Param " + i + ": " 
		     + Util.pad_left(f10_5.format(param_val[i]),11));
    }

  }



/*
   log2:  make up for a weak math library with only ln(x)
*/

static float log2(float x)
    {
    return (float) (Math.log(x)/0.6931472);  // log2(x)=ln(x)/ln(2) 
    }


}
