/*       Copyrighted by Kenneth De Jong - 2005          */
/* Licensed under the Academic Free License version 2.1 */

/*
   EA:  base class for simple evolutionary algorithms
*/

import java.util.*;
import java.text.DecimalFormat;


abstract class EA
    {

    String  ea_type;
    boolean use_recombination = true;
    boolean use_mutation = true;
    EC_rand  rand;

    Population parent_pop;
    int parent_pop_size;

    Population offspring_pop;
    int offspring_pop_size;
    int brood_size;

    int birth_cnt = 0;
    int birth_limit;
    int death_cnt = 0;
    int generation_cnt = 0;

    Individual   global_max;
    Individual   global_min;
    float  global_ave = 0;

    boolean report_global_stats = true;
    boolean report_local_stats = true;
    boolean report_pop_dump = true;
    boolean report_pop_stats = false;
    boolean report_bsf = false;

    abstract void run(int birth_limit);    // run EA for specified limit


/*
   EA constructor
*/

EA(int rand_seed)
    {  
    rand = new EC_rand(rand_seed);
    }



/*
   EA_init: finish general EA initialization
*/

void EA_init(Properties props)
    {
    String report_type;

    System.out.println("  Parent population size: " + parent_pop_size +
		       "; offspring population size: " + offspring_pop_size);

    report_type = props.getProperty("Report");
    if (report_type == null) {
	Util.ec_warn("Report type not specified; default used.");
	report_type = "full";
        }

    if (report_type.equals("local"))
        {
	report_global_stats = false;
	report_pop_dump = false;
        }
    else if (report_type.equals("global"))
        {
	report_local_stats = false;
	report_pop_dump = false;
        }
    else if (report_type.equals("bsf"))
	{
        report_bsf = true;
	report_global_stats = false;
	report_local_stats = false;
	report_pop_dump = false;
        }
    else if (report_type.equals("pop_stats"))
	{
	report_pop_stats = true;
	report_local_stats = true;
	report_pop_dump = false;
        report_bsf = false;
	report_global_stats = false;
        }

    if (report_pop_stats)
	System.out.println("     births   max_fit     ave_fit     min_fit");

    dump();
    }


DecimalFormat int4 = new DecimalFormat("###0");
DecimalFormat f12_5 = new DecimalFormat("#####0.00000");
DecimalFormat f12_6 = new DecimalFormat("#####0.000000");


/*
  dump(): prints out current population & related stats
*/

void dump()
  { 
  int i;

  if (report_bsf)
      {
      System.out.println(birth_cnt + "  " + global_max.fitness);
      }
  else if (!report_pop_stats)
      {
      System.out.println("");

      generation_cnt = birth_cnt / parent_pop.pop_size;
      System.out.println("Population data after " + birth_cnt + 
			 " births (generation " + generation_cnt + "):");
      }

  if (report_global_stats)
      {
      global_max.genome.global_stats();

      System.out.println(
                " Global fitness:  max = " 
		+ Util.pad_left(f12_5.format(global_max.fitness),9) +
		", ave = " +
		Util.pad_left(f12_5.format(global_ave/birth_cnt),9) + 
                ", min = " +
		Util.pad_left(f12_5.format(global_min.fitness),9) );
      }
  
  if (report_local_stats)  parent_pop.dump();

  }




/*
   Print_best:
*/

void print_best()
  {
  System.out.println("\nGlobal best:  birthdate   fitness      gene values"); 

  System.out.print("           " + 
	  Util.pad_left(int4.format(global_max.birthdate), 7) + 
	  Util.pad_left(f12_6.format(global_max.fitness), 16) + "  ");
  global_max.genome.print();

  System.out.println("");
  }

}


