/*       Copyrighted by Kenneth De Jong - 2005          */
/* Licensed under the Academic Free License version 2.1 */

/*
    EP_EA:  a simple Evolutionary Programming EA
*/

import java.util.*;


class EP_EA   extends EA
    {


/*
    EP_EA constructor:
*/

EP_EA(Properties props, Landscape lscape, int rand_seed)
    {
    super(rand_seed);

    String sizes;
    String representation;
    StringTokenizer  st;

    ea_type = "ep";
    parent_pop_size = 10;

    sizes = props.getProperty("PopSize");
    if (sizes == null)
	Util.ec_warn("PopSize not specified; using defaults.");
    else {
	st = new StringTokenizer(sizes," ");
	parent_pop_size = Integer.parseInt(st.nextToken());
        }
    offspring_pop_size = parent_pop_size;

    representation = props.getProperty("Representation");
    if (!(representation == null)) {
	if (! representation.equals("real"))
	    Util.ec_error("unknown EP representation type: " + representation);
        }
    RealRepresentation.RealRepresentationInit(props, (RvpLscape)lscape, (EA)this);

    EA_init(props);         // finish general EA initialization;
    }


/*
   run: an EP-style EA
       
      - produces an offspring population by mutating and/or recombining 
        one copy of each parent.

      - injects offspring population into parent population via 
        truncation selection.
*/

void run(int birth_limit)
    {
    while (birth_cnt < birth_limit)
	{
        offspring_pop = parent_pop.copy(1);
	if (use_recombination)  offspring_pop.mate_unif(parent_pop);
	if (use_mutation)  offspring_pop.mutate();
	offspring_pop.evaluate();
	parent_pop.truncation_merge(offspring_pop);

	if ((birth_cnt % parent_pop.pop_size) == 0)  dump();
	}

    print_best();
    }

}

