/*       Copyrighted by Kenneth De Jong - 2005          */
/* Licensed under the Academic Free License version 2.1 */

/*
    ES_EA:  a simple Evolution Strategy
*/

import java.util.*;


class ES_EA   extends EA
    {


/*
    ES_EA constructor
*/

ES_EA(Properties props, Landscape lscape, int rand_seed)
    {
    super(rand_seed);

    String sizes;
    String representation;
    StringTokenizer  st;

    ea_type = "es";
    parent_pop_size = 10;
    brood_size = 1;

    sizes = props.getProperty("PopSize");
    if (sizes == null)
	Util.ec_warn("PopSize not specified; using defaults.");
    else {
	st = new StringTokenizer(sizes," ");
	parent_pop_size = Integer.parseInt(st.nextToken());
	if (st.hasMoreTokens()) 
	    brood_size = Integer.parseInt(st.nextToken());
        }
    offspring_pop_size = parent_pop_size * brood_size;

    representation = props.getProperty("Representation");
    if (!(representation == null)) {
	if (! representation.equals("real"))
	    Util.ec_error("unknown EP representation type: " + representation);
        }
    RealRepresentation.RealRepresentationInit(props, (RvpLscape)lscape, (EA)this);

    EA_init(props);         // finish general EA initialization;
    }


/*
    run: an ES-style EA
       
      - produces an offspring population by mutating B copies of each parent
      - injects offspring population into parent population via truncation selection
*/

void run(int birth_limit)
    {
    float mutation_success_rate;

    while (birth_cnt < birth_limit)
	{

        offspring_pop = parent_pop.copy(brood_size);
	if (use_recombination)  offspring_pop.mate_unif(parent_pop);
	if (use_mutation)  offspring_pop.mutate();
	offspring_pop.evaluate();
	parent_pop.truncation_merge(offspring_pop);

	if (RealOperators.adaptive_mutation)
	    {
 	    mutation_success_rate = (float) offspring_pop.reproductive_success 
		                  / (float) offspring_pop.pop_size;
            if (mutation_success_rate > 0.21F)  
		RealOperators.mutation_step_size = (1+RealOperators.adaptive_step_size) 
		                           * RealOperators.mutation_step_size;
	    else if (mutation_success_rate < 0.19F)  
		RealOperators.mutation_step_size = (1-RealOperators.adaptive_step_size) 
		                           * RealOperators.mutation_step_size;

	    }

	if ((birth_cnt % parent_pop.pop_size) == 0)  dump();

        if (RealOperators.adaptive_mutation && report_global_stats)
	  {
	  System.out.println("  Reproductive success: " + 
			      offspring_pop.reproductive_success);
	  System.out.println("  Mutation step size: " + 
			      RealOperators.mutation_step_size);
	  }
	}

    print_best();
    }

}

