/*       Copyrighted by Kenneth De Jong - 2005          */
/* Licensed under the Academic Free License version 2.1 */

/*
    EV_EA:  the simplest evolutionary algorithm
*/

import java.util.*;


class EV_EA   extends EA
    {

/*
    EV_EA constructor
*/

EV_EA(Properties props, Landscape lscape, int rand_seed)
    {
    super(rand_seed);

    String sizes;
    String representation;
    StringTokenizer  st;

    ea_type = "ev";
    parent_pop_size = 10;
    offspring_pop_size = 1;

    sizes = props.getProperty("PopSize");
    if (sizes == null)
	Util.ec_warn("PopSize not specified; default used.");
    else {
	st = new StringTokenizer(sizes," ");
        parent_pop_size = Integer.parseInt(st.nextToken());

	if (st.hasMoreTokens()) 
	    offspring_pop_size = Integer.parseInt(st.nextToken());
	else
	    Util.ec_warn("offspring population size not specified; default used.");
        }

    representation = props.getProperty("Representation");
    if (!(representation == null)) {
	if (! representation.equals("real"))
	    Util.ec_error("unknown EV representation type: " + representation);
        }
    RealRepresentation.RealRepresentationInit(props, (RvpLscape) lscape, (EA)this);

    EA_init(props);         // finish general EA initialization;
    }


/*
   run: an EV-style EA
       
      - produces an offspring population from (uniform) randomly selected parents
        using mutation and/or crossover.

      - injects offspring population into parent population via uniform binary
        tournament selection.
*/

void run(int birth_limit)
    {
    while (birth_cnt < birth_limit)
	{
        offspring_pop = parent_pop.select_urand(offspring_pop_size);
	if (use_recombination)  offspring_pop.mate_unif(parent_pop);
	if (use_mutation) offspring_pop.mutate();
	offspring_pop.evaluate();
	parent_pop.urand_binary_tourn_merge(offspring_pop);

	if ((birth_cnt % parent_pop.pop_size) == 0)  dump();
	}

    print_best();
    }

}

