/*       Copyrighted by Kenneth De Jong - 2005          */
/* Licensed under the Academic Free License version 2.1 */

/*
    GA_EA:  a simple Genetic Algorithm
*/

import java.util.*;


class GA_EA   extends EA
    {


/*
    GA_EA constructor
*/

GA_EA(Properties props, Landscape lscape, int rand_seed)
    {
    super(rand_seed);

    String sizes;
    String representation;
    StringTokenizer  st;

    ea_type = "ga";
    parent_pop_size = 10;  // default

    sizes = props.getProperty("PopSize");
    if (sizes == null)
	Util.ec_warn("PopSize not specified; using defaults.");
    else {
	st = new StringTokenizer(sizes," ");
	parent_pop_size = Integer.parseInt(st.nextToken());
        }
    offspring_pop_size = parent_pop_size;

    representation = props.getProperty("Representation");
    if (representation == null) {
	Util.ec_warn("Representation type unspecified; using default.");
	representation = "binary";
        }

    if (representation.equals("binary"))
	BinaryRepresentation.BinaryRepresentationInit(props, (RvpLscape)lscape, (EA)this);
    else if (representation.equals("real"))
        RealRepresentation.RealRepresentationInit(props, (RvpLscape)lscape, (EA)this);
    else
	Util.ec_error("unknown GA representation type: " + representation);

    EA_init(props);         // finish general EA initialization;
    }


/*
    run: a GA-style EA
       
      - produces an offspring population by:
           - cloning M parents selected by fitness proportional selection
           - mate and mutate the clones
	   - replacing the parent population with the children
*/

void run(int birth_limit)
    {

    while (birth_cnt < birth_limit)
	{

        offspring_pop = parent_pop.select_fprop(offspring_pop_size);
	if (use_recombination)  offspring_pop.mate_fprop(parent_pop);
	if (use_mutation)  offspring_pop.mutate();
	offspring_pop.evaluate();
	parent_pop.indiv = offspring_pop.indiv;

	if ((birth_cnt % parent_pop.pop_size) == 0)  dump();
	}

    print_best();
    }

}

