/*       Copyrighted by Kenneth De Jong - 2005          */
/* Licensed under the Academic Free License version 2.1 */

/*
   RvpEA_application:  apply a specific EA to specific real-valued
                       parameter optimization landscape.
*/

import java.util.*;


class RvpEA_application  extends EA_application
    {

/*
   RvpEA_application constructor:
*/

RvpEA_application(StringTokenizer lscape_props, Properties props)
    {
    super(props);
   
    String lscape_name;
    String ea_type;

    if (! lscape_props.hasMoreTokens())
	Util.ec_error("missing landscape name.");

    lscape_name = lscape_props.nextToken();
    if (! lscape_name.equals("Parabola"))
	Util.ec_error("unknown landscape: " + lscape_name);

    landscape = new Parabola(lscape_props);
    landscape.print_info();

    ea_type = props.getProperty("EA");
    if (ea_type == null) {
	Util.ec_warn("no EA type specified; using default.");
	ea_type = "ev";
        }
    System.out.println("  EA type: " + ea_type);

    if (ea_type.equals("ev"))
       ea = new EV_EA(props, (RvpLscape) landscape, rand_seed);
    else if (ea_type.equals("ep"))
       ea = new EP_EA(props, (RvpLscape) landscape, rand_seed);
    else if (ea_type.equals("es"))
       ea = new ES_EA(props, (RvpLscape) landscape, rand_seed);
    else if (ea_type.equals("ga"))
       ea = new GA_EA(props, (RvpLscape) landscape, rand_seed);
    else 
	Util.ec_error("unknown EA type: " + ea_type);
    }


/*
  run: runs the given EA on the given landscape
*/

void run()
  {
  ea.run(simulation_limit);
  }

}
