/*       Copyrighted by Kenneth De Jong - 2005          */
/* Licensed under the Academic Free License version 2.1 */

/*
  2D Rosenbrock function
*/

import java.util.*;


class Rosenbrock extends RvpLscape
  {

  float fitness_offset = 0;

/*
   Rosenbrock Constructor
*/

Rosenbrock(StringTokenizer st)
    {

    int  i;
    String  token;
    float   param_lbd, param_ubd;
    String  firmness;

    if (st.hasMoreTokens())   // landscape dimension
	{
	token = st.nextToken();
	param_cnt = Integer.parseInt(token);
	};

    param_cnt = 2;   // only default implemented for now

    if (st.hasMoreTokens())   // parameter lower bound
	{
	token = st.nextToken();
	param_lbd = Float.parseFloat(token);
	}
    else  param_lbd = -4.0F;   //default

    if (st.hasMoreTokens())   // parameter upper bound
	{
	token = st.nextToken();
	param_ubd = Float.parseFloat(token);
	}
    else  param_ubd = 4.0F;   //default

    if (st.hasMoreTokens())   // hard/soft bounds
	{
	firmness = st.nextToken();
	}
    else  firmness = "hard";   //default


    init_param_lbd = new float[param_cnt];
    init_param_ubd = new float[param_cnt];
    init_param_res = new float[param_cnt];

    hard_param_lbd = new float[param_cnt];
    hard_param_ubd = new float[param_cnt];

    for (i=0; i<param_cnt; i++)
	{
	init_param_lbd[i] = param_lbd;
        init_param_ubd[i] = param_ubd;
	init_param_res[i] = 0.0001F;

	if (firmness.equals("hard"))
	    {
	    hard_param_lbd[i] = param_lbd;
            hard_param_ubd[i] = param_ubd;
	    }
	else
	    {
            hard_param_lbd[i] = Float.NEGATIVE_INFINITY;
            hard_param_ubd[i] = Float.POSITIVE_INFINITY;
	    }
	}

    if (st.hasMoreTokens())   // inversion
	{
	token = st.nextToken();
        if (token.equals("-"))  inverted = true;

	if (st.hasMoreTokens())   // Fitness offset
	   {
	   token = st.nextToken();
	   fitness_offset = Float.parseFloat(token);
	   }
	}
    }


void print_info()
{
  int i;

  if (inverted)
      System.out.println("  Using an invertewd Rosenbrock landscape defined on "
	     + param_cnt + " parameter(s) with offset " + fitness_offset);
  else
      System.out.println("  Using a Rosenbrock landscape defined on "
	     + param_cnt + " parameter(s)");

  System.out.println("   with parameter initialization bounds of: ");

  for (i=0; i<param_cnt; i++)
     System.out.println("     " + (i+1) + ": " + init_param_lbd[i] + 
	       "  " + init_param_ubd[i]);

  System.out.println("   and with hard parameter bounds of: ");

  for (i=0; i<param_cnt; i++)
     System.out.println("     " + (i+1) + ": " + hard_param_lbd[i] + 
	       "  " + hard_param_ubd[i]);
}



/*
    eval:  computes fitness of a genome for the given landscape
*/

float eval(float param[])
    {
    float fitness = 0;
    float term1, term2;

    term1 = (param[0]*param[0])-param[1];
    term2 = 1-param[0];

    fitness = 100*(term1*term1) + (term2*term2);

    if (inverted)  return (fitness_offset-fitness);
    else   return(fitness);
    }



/*
    legal:  checks legality of the parameter values
*/

boolean legal(float param[])
    {
    boolean validity = true;
    int i;

    for (i=0; i<param_cnt; i++)
       if ( (param[i] < hard_param_lbd[i]) || (param[i] > hard_param_ubd[0]) )
	   validity = false;

    return(validity);
    }


}
