/*       Copyrighted by Kenneth De Jong - 2005          */
/* Licensed under the Academic Free License version 2.1 */

/*
    EA_application:  base class for applying a specific EA to a
                     specific landscape.
*/

import java.util.*;


abstract class EA_application
  {

  EA  ea;
  Landscape landscape;
  int simulation_limit;
  int rand_seed;
  int run_count;

  abstract void run();


/*
   EA_application constructor:
*/

EA_application(Properties props)
  {
  String seed;
  String limit;
  String report_options;
  StringTokenizer st;
  String token;
  String runcount;

  limit = props.getProperty("SimLimit");
  if (limit == null) {
      Util.ec_warn("no SimLimit specified; default used.");
      simulation_limit = 100;
      }
  else simulation_limit = Integer.parseInt(limit);
  System.out.println("  Simulation time limit (# births): " + simulation_limit);

  seed = props.getProperty("RandSeed");
  if (seed == null) {
      Util.ec_warn("no RandSeed specified; default used.");
      rand_seed = 12345;
      }
  else rand_seed = Integer.parseInt(seed);
  System.out.println("  Random number seed (positive integer): " + rand_seed);
 
  runcount = props.getProperty("RunCount");
  if (runcount == null) { 
      Util.ec_warn("no RunCount specified; default used.");
      run_count = 1;
      }
  else run_count = Integer.parseInt(runcount);

  System.out.println("  RunCount: " + run_count);
  }


/*
   EA_applicationInit:
*/

void EA_applicationInit(Properties props)
    {  
    String ea_type;

    ea_type = props.getProperty("EA");
    if (ea_type == null) {
	Util.ec_warn("no EA type specified; using default.");
	ea_type = "UN";
        }
        
    System.out.println("  EA type: " + ea_type);
    if (ea_type.equals("EV"))
       ea = new EV_EA(props, landscape, rand_seed);
    else if (ea_type.equals("EP"))
       ea = new EP_EA(props, landscape, rand_seed);
    else if (ea_type.equals("ES"))
       ea = new ES_EA(props, landscape, rand_seed);
    else if (ea_type.equals("GA"))
       ea = new GA_EA(props, landscape, rand_seed);
    else if (ea_type.equals("UN"))
       ea = new UN_EA(props, landscape, rand_seed);
    else 
	Util.ec_error("unknown EA type: " + ea_type);
    }

}
