/*       Copyrighted by Kenneth De Jong - 2005          */
/* Licensed under the Academic Free License version 2.1 */

/*
    ES_EA:  a simple Evolution Strategy
*/

import java.util.*;


class ES_EA   extends EA
    {


/*
    ES_EA constructor
*/

ES_EA(Properties props, Landscape lscape, int rand_seed)
    {
    super(rand_seed);

    String sizes;
    StringTokenizer  st;

    ea_type = "ES";
    parent_pop_size = 1;  // defaults
    brood_size = 10;

    representation_name = "real";
    representation_type = real_rep;

    parent_selection_name = "uniform_deterministic";
    parent_selection_type = uniform_deterministic;
    mate_selection_type = uniform_stochastic;

    generation_name = "overlapping";
    generation_type = overlapping;

    survival_selection_name = "truncation";
    survival_selection_type = truncation;


    EA_init(props, lscape);         // finish general EA initialization;
    }


/*
  adapt_operators:  dynamically adjust operator parameters
*/

void adapt_operators()
    {
    float mutation_success_rate;

    if (RealGenome.adaptive_mutation) {
 	mutation_success_rate = (float) offspring_pop.reproductive_success 
	                  / (float) offspring_pop.pop_size;
        if (mutation_success_rate > 0.21F)  
	    RealGenome.mutation_step_size = (1+RealGenome.adaptive_step_size) 
		                          * RealGenome.mutation_step_size;
	else if (mutation_success_rate < 0.19F)  
	    RealGenome.mutation_step_size = (1-RealGenome.adaptive_step_size) 
		                          * RealGenome.mutation_step_size;
        }

    }


/*
  Adaptation_stats:
*/

void adaptation_stats()
  {
  if (RealGenome.adaptive_mutation && report_global_stats)
      {
      System.out.println("  Reproductive success: " +  offspring_pop.reproductive_success);
      System.out.println("  Mutation step size: " +  RealGenome.mutation_step_size);
      }

  }

}
