/*       Copyrighted by Kenneth De Jong - 2005          */
/* Licensed under the Academic Free License version 2.1 */

/*
  Graph2r landscape: N x N connectivity matrices with odd numbered row constraints
*/

import java.util.*;


class Graph2r extends IvpLscape
  {

  int dimension;
  int     param_lbd, param_ubd;
  int     payoff[];

/*
   Graph2r Constructor
*/

Graph2r(StringTokenizer st)
    {

    int  i;
    String  token;

    lscape_type = "IVP";
    lscape_name = "Graph2r";

    if (st.hasMoreTokens())   // number of nodes in the graph
	{
	token = st.nextToken();
	dimension = Integer.parseInt(token);
	}
    else  dimension = 10;   //default

    if (st.hasMoreTokens())   // parameter lower bound
	{
	token = st.nextToken();
	param_lbd = Integer.parseInt(token);
	}
    else  param_lbd = 0;   //default

    if (st.hasMoreTokens())   // parameter upper bound
	{
	token = st.nextToken();
	param_ubd = Integer.parseInt(token);
	}
    else  param_ubd = 1;   //default

    param_cnt = dimension * dimension;
    init_param_lbd = new int[param_cnt];
    init_param_ubd = new int[param_cnt];
    payoff = new int[param_cnt];

    for (i=0; i<param_cnt; i++)
	{
	init_param_lbd[i] = param_lbd;
        init_param_ubd[i] = param_ubd;
	if (((i/dimension) % 2) == 0)  payoff[i] = 10;
	else payoff[i] = -10;
	}

    }


void print_info()
{
  int i;

  System.out.println("  Using a Graph2r landscape defined on "
	     + param_cnt + " parameter(s)");

  System.out.println("   with parameter bounds of: " + param_lbd + "," + param_ubd);
  
  System.out.println("   Associated payoff matrix is: ");
  for (i=0; i<param_cnt; i++) System.out.print(payoff[i] + " ");
  

}



/*
    eval:  computes fitness of a genome for the given landscape
*/

float eval(int param[])
    {
    float fitness = 0;
    int i;

    if (legal(param)) {
	for (i=0; i<param_cnt; i++)  {
	    if (param[i] == 1)  fitness += payoff[i];
	    if (i > 0) 
		if (param[i-1] == 1) fitness += payoff[i-1];
	    if (i < param_cnt-1) 
		if (param[i+1] == 1) fitness += payoff[i+1];
	    }
        return(fitness);
	}
    else return(-(dimension*dimension*dimension));   // constraint violation
    }



/*
    legal:  checks legality of the parameter values
*/

boolean legal(int param[])
    {
    boolean validity = true;
    int i;

    for (i=0; i<param_cnt; i++)
	if ((param[i] < param_lbd) || (param[i] > param_ubd)) {
	    validity = false;
	    break;
	    }

    return(validity);
    }


}
