/*       Copyrighted by Kenneth De Jong - 2005          */
/* Licensed under the Academic Free License version 2.1 */

/*
  N-sum landscape:
*/

import java.util.*;


class Nsum extends IvpLscape
  {


/*
   Nsum Constructor
*/

Nsum(StringTokenizer st)
    {

    int  i;
    String  token;
    int   param_lbd, param_ubd;

    lscape_type = "IVP";
    lscape_name = "Nsum";

    if (st.hasMoreTokens())   // landscape dimension
	{
	token = st.nextToken();
	param_cnt = Integer.parseInt(token);
	}
    else  param_cnt = 10;   //default

    if (st.hasMoreTokens())   // parameter lower bound
	{
	token = st.nextToken();
	param_lbd = Integer.parseInt(token);
	}
    else  param_lbd = 0;   //default

    if (st.hasMoreTokens())   // parameter upper bound
	{
	token = st.nextToken();
	param_ubd = Integer.parseInt(token);
	}
    else  param_ubd = 1;   //default


    init_param_lbd = new int[param_cnt];
    init_param_ubd = new int[param_cnt];

    for (i=0; i<param_cnt; i++)
	{
	init_param_lbd[i] = param_lbd;
        init_param_ubd[i] = param_ubd;
	}

    }


void print_info()
{
  int i;

  System.out.println("  Using an Nsum landscape defined on "
	     + param_cnt + " parameter(s)");

  System.out.println("   with parameter bounds of: ");

  for (i=0; i<param_cnt; i++)
     System.out.println("     " + (i+1) + ": " + init_param_lbd[i] + 
	       "  " + init_param_ubd[i]);

}



/*
    eval:  computes fitness of a genome for the given landscape
*/

float eval(int param[])
    {
    float fitness = 0;
    int i;

    for (i=0; i<param_cnt; i++)  fitness += param[i];

    return(fitness);
    }



/*
    legal:  checks legality of the parameter values
*/

boolean legal(int param[])
    {
    boolean validity = true;
    int i;

    for (i=0; i<param_cnt; i++)
       if ( (param[i] < init_param_lbd[i]) || (param[i] > init_param_ubd[0]) )
	   validity = false;

    return(validity);
    }


}
