/*       Copyrighted by Kenneth De Jong - 2005          */
/* Licensed under the Academic Free License version 2.1 */

/*
   Genome:   a simple real-valued parameter genome representation 
*/


import java.text.DecimalFormat;


class Genome {

    static RvpLscape rls;
    static RvpEA    ea;

    static int   gene_cnt;
    static float gene_lbd[];
    static float gene_ubd[];

    static float mutation_step_sum=0;
    static int   mutation_count=0;

    float gene[];

    float mutation_prob = 1.0F / gene_cnt;
    boolean valid_genome = true;

/*
   Genome constructor: uses a uniform random number generator to assign
                initial values to each gene from the range specified 
		by the the given lower and upper bounds.
*/

Genome(EC_rand rand)
  {
  int  i;

  gene = new float[gene_cnt];

  for (i=0; i<gene_cnt; i++)
    gene[i] = gene_lbd[i] + (gene_ubd[i] - gene_lbd[i]) * rand.frand();
  }



/*
   Genome Constructor:
*/

Genome()
  {
  int  i;

  gene = new float[gene_cnt];

  }




/*
   map_genome: initializes the genome/phenome mapping for entire class
*/

static void map_genome(RvpLscape lscape) {

  int i;

  rls = lscape;

  gene_cnt = rls.param_cnt;
  gene_lbd = new float[gene_cnt];
  gene_ubd = new float[gene_cnt];

  for (i=0; i<gene_cnt; i++) {
      gene_lbd[i] = rls.init_param_lbd[i];
      gene_ubd[i] = rls.init_param_ubd[i];
      }

  System.out.println("  Using a genome with " + gene_cnt +
		     " real-valued gene(s).");
  }


/*
   eval: computes the fitness of a given genome
*/

float eval()
  {
  return (rls.eval(gene));
  }


/*
  Mutation:
*/

void mutate(EC_rand rand, int mutation_type, float mutation_step_size)
  {
  switch (mutation_type) {
      case RvpEA.no_mutation:
	  break;

      case RvpEA.delta_mutation:
	  delta_mutate(rand, mutation_step_size);
	  break;

      case RvpEA.gaussian_mutation:
	  gaussian_mutate(rand, mutation_step_size);
	  break;

      default:
	  Util.ec_error("illegal mutation type: " + mutation_type);
      }

  return;
  }

/*
   delta_mutate: mutates a given genome 

       - each gene has a 1/L chance of being mutated, where L is the 
         number of genes in the genome.

       - the selected genes are mutated by incrementing/decrementing
         their values by a fixed delta.

*/
            
void delta_mutate(EC_rand rand, float mutation_delta)
  {
  float  new_val[];
  int    i;
  int mutated = 0;

  new_val = new float[gene_cnt];

  for (i=0; i<gene_cnt; i++)
    {
    if (rand.frand() <= mutation_prob)
      {
      if (rand.frand() < 0.5F)  new_val[i] = gene[i] + mutation_delta;
      else  new_val[i] = gene[i] - mutation_delta;
      mutation_step_sum += mutation_delta;
      mutated++;
      }
    else  new_val[i] = gene[i];
    }

  if (mutated>0)
     if (rls.legal(new_val))  gene = new_val;

  mutation_count += mutated;
  return;
  }




/*
   Gaussian_mutate: mutates a given genome 

       - each gene has a 1/L chance of being mutated, where L is the 
         number of genes in the genome.

       - the selected genes are mutated by incrementing/decrementing
         their values by a Gaussian delta.

*/
            
void gaussian_mutate(EC_rand rand, float mutation_delta)
  {
  float  new_val[];
  int    i;
  int mutated = 0;

  new_val = new float[gene_cnt];

  for (i=0; i<gene_cnt; i++)
    {
    if (rand.frand() <= mutation_prob)
	{                  // ave_step_size = (2/sqrt(2pi))*sigma
      new_val[i] = gene[i] + 1.3F*mutation_delta*rand.gaussian();
      mutation_step_sum += Math.abs(new_val[i]-gene[i]);
      mutated++;
      }
    else  new_val[i] = gene[i];
    }

  if (mutated>0)
     if (rls.legal(new_val))  gene = new_val;

  mutation_count += mutated;
  return;
  }



DecimalFormat f12_5 = new DecimalFormat("#####0.00000");


/*
   print_genome:  prints the gene values as real numbers
*/

void print()
  {
  int  i;

  for (i=0; i<gene_cnt; i++)
        System.out.print(Util.pad_left(f12_5.format(gene[i]), 13));
  }



/*
   copy:
*/

Genome copy()
  {
  Genome  dupl = new Genome();

  dupl.gene_cnt = gene_cnt;
  dupl.gene = new float[gene_cnt];

  System.arraycopy(gene, 0, dupl.gene, 0, gene_cnt);

  return (dupl);
  }

}



