/*       Copyrighted by Kenneth De Jong - 2005          */
/* Licensed under the Academic Free License version 2.1 */

/*
   Genome:   base class for genome types
*/

import java.util.*;


abstract class Genome
    {

    static int    gene_cnt;

    static int genotype_cardinality;
    static int        initialization_type;
    static final int  one_genome = 1;
    static final int  two_genomes = 2;
    static final int  random   = 3;
    static boolean    init_high = false;

    static boolean use_recombination = true;
    static int        crossover_type;
    static final int  one_point = 1;
    static final int  two_point = 2;
    static final int  uniform   = 3;
    static final int  blend = 4;
    static float uniform_crossover_probability = 0.5F;
    static float blend_ratio = 0.5F;
    static boolean  keep_both_sides = false;

    static boolean    use_mutation = true;
    static int        mutation_type;
    static final int  bitflip_mutation = 1;
    static final int  delta_mutation = 2;
    static final int  gaussian_mutation = 3;
    static final int  random_mutation = 4;

    static int        mutation_count=0;
    static float      mutation_probability;

    static float      mutation_step_sum = 0;
    static float      invalid_mutation_count = 0;

    abstract void   run_reset(EC_rand rand);
    abstract void   mutate(EA ea);
    abstract void   crossover(Genome mate, EC_rand rand);
    abstract float  eval();
    abstract Genome copy();
    abstract void   print();
    abstract void   global_stats();
    abstract float  distance(Genome partner);


static void GenomeClassInit(Properties props, EA ea)
    {
    String  crossover_options;
    String  crossover_spec;
    StringTokenizer  st;

    crossover_options = props.getProperty("Crossover");
    if (crossover_options == null) {
	if (ea.ea_type.equals("ga")) {
	    Util.ec_warn("Crossover type unspecified; using default.");
	    crossover_spec = "one_point";
	    }
	else {
	    crossover_spec = "none";
	     use_recombination = false;
	     ea.use_recombination = false;
	     }
        }
    else {
		st = new StringTokenizer(crossover_options," ");
		crossover_spec = st.nextToken();

		if (crossover_spec.equals("none")) {
			use_recombination = false;
			ea.use_recombination = false;
            }
		else if (crossover_spec.equals("one_point")) crossover_type = one_point;
		else if (crossover_spec.equals("two_point")) crossover_type = two_point;
		else if (crossover_spec.equals("uniform")) {
			crossover_type = uniform;
			if (st.hasMoreTokens())
			uniform_crossover_probability = Float.parseFloat(st.nextToken());
                        }
		else if (crossover_spec.equals("blend")) {
			crossover_type = blend;
			if (st.hasMoreTokens())
			blend_ratio = Float.parseFloat(st.nextToken());
                        }
		else Util.ec_error("unknown crossover type: " + crossover_spec);

		if (st.hasMoreTokens()) {
			crossover_spec = st.nextToken();
			if (crossover_spec.equals("keep_both_sides"))  keep_both_sides = true;
                        else Util.ec_error("Unknown crossover option: " + crossover_spec);
			}
		}
		
    if (crossover_spec.equals("uniform"))
	System.out.println("  Crossover: " + crossover_spec + 
			   " " + uniform_crossover_probability);
    else if (crossover_spec.equals("blend"))
	System.out.println("  Crossover: " + crossover_spec + 
			   " " + blend_ratio);
    else
	System.out.println("  Crossover: " + crossover_spec);
	
    String  initialization_options;
    String  initialization_spec;

    initialization_options = props.getProperty("PopulationInitialization");
    if (initialization_options == null) {
            Util.ec_warn("population initialization type unspecified; using the default (random).");
	    initialization_spec = "random";
	    }
    else {
            st = new StringTokenizer(initialization_options," ");
            initialization_spec = st.nextToken();
            }
		
    if (initialization_spec.equals("random")) initialization_type = random;
    else if (initialization_spec.equals("one_genome")) initialization_type = one_genome;
    else if (initialization_spec.equals("two_genomes")) initialization_type = two_genomes;
    else Util.ec_error("unknown population initialization type: " + initialization_spec);

    }


static void Genome_run_reset()
    {
    mutation_count = 0;
    mutation_step_sum = 0;
    invalid_mutation_count = 0;
    }

}



